VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "F4"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Private f4mem(65535) As Long
Private A As Long
Private PC As Long
Private C As Long
Private V As Boolean
Private HighestMem As Long

Private Sub Class_Initialize()
'Wash memory for this processor, and set accumulator, PC,
' and overflow flags to initial state (0 vector)
Dim n As Long
  For n = 0 To 65535
    f4mem(n) = 0
  Next n
  HighestMem = 0
  C = 0
  A = 0
  PC = 0
  V = False
End Sub

Private Sub Class_Terminate()
'No special code for termination yet
End Sub

Property Get Overflow() As Boolean
  Overflow = V
End Property

Property Get Accumulator() As Long
  Accumulator = A
End Property

Property Get PCounter() As Long
  PCounter = PC
End Property

Property Get Cycles() As Long
  Cycles = C
End Property

Property Get HiMem() As Long
  HiMem = HighestMem
End Property


Public Sub Load_Ram(filename As String)
'Load a program rom file to memory
Dim opc As String
Dim oper As String
Dim tline As String
Dim temp As Long
Dim n As Long


  On Error GoTo ErrorHandler   ' Enable error-handling routine.
  
  Open filename For Input Access Read As #1
  'Check for error opening file
  
  HighestMem = 0
  While EOF(1) = False
    Input #1, tline
    If Mid$(tline, 1, 1) <> "{" Then
      'decode - quite crude for now but good enough to experiment with
      'Find the first space character in the line
      For n = 1 To Len(tline)
        If Mid$(tline, n, 1) = Chr(32) Then Exit For
      Next n
      If n < Len(tline) Then
        opc = Mid$(tline, 1, n - 1)
        oper = Mid$(tline, n + 1, Len(tline) - n)
        temp = Round(Val(oper))
        If opc = "ADDm" Then f4mem(HighestMem) = 1
        If opc = "ADDi" Then f4mem(HighestMem) = 2
        If opc = "ADDpc" Then f4mem(HighestMem) = 4
        If opc = "BVS" Then f4mem(HighestMem) = 8
        If opc = "LDAm" Then f4mem(HighestMem) = 16
        If opc = "LDAi" Then f4mem(HighestMem) = 32
        If opc = "LDApc" Then f4mem(HighestMem) = 64
        If opc = "STAm" Then f4mem(HighestMem) = 128
        If opc = "STApc" Then f4mem(HighestMem) = 256
        HighestMem = HighestMem + 1
        f4mem(HighestMem) = temp
        HighestMem = HighestMem + 1
      End If
    End If
  Wend
  Close #1

  On Error GoTo 0   ' Turn off error trapping.

  Exit Sub

' Error-handling routine.
ErrorHandler:
   Select Case Err.Number
      Case 55   ' "File already open" error.
         Close #1   ' Close open file.
      Case Else
         'Search docs for Errors,numbers to add more stuff
   End Select
   Resume
        

End Sub

Public Function Disassemble(Location As Long)
'Disassembles starting at a specific memory location
'Returns mnemonic strings
Dim buildstr As String
 
  buildstr = CStr(Location) + "  "
  Select Case f4mem(Location)
    Case 1
      buildstr = buildstr + " ADDm " + CStr(f4mem(Location + 1))
    Case 2
      buildstr = buildstr + " ADDi " + CStr(f4mem(Location + 1))
    Case 4
      buildstr = buildstr + " ADDpc "
    Case 8
      buildstr = buildstr + " BVS " + CStr(f4mem(Location + 1))
    Case 16
      buildstr = buildstr + " LDAm " + CStr(f4mem(Location + 1))
    Case 32
      buildstr = buildstr + " LDAi " + CStr(f4mem(Location + 1))
    Case 64
      buildstr = buildstr + " LDApc "
    Case 128
      buildstr = buildstr + " STAm " + CStr(f4mem(Location + 1))
    Case 256
      buildstr = buildstr + " STApc " + CStr(f4mem(Location + 1))
    
   Case Else
     buildstr = buildstr + " ??? "
  End Select
  Disassemble = buildstr

End Function

Public Sub Step()
'executes instruction pointed to by program counter
  Select Case f4mem(PC)
    Case 1
      Call Add_Amem(f4mem(PC + 1))
    Case 2
      Call Add_Aimm(f4mem(PC + 1))
    Case 4
      Call Add_APC
    Case 8
      Call BVS(f4mem(PC + 1))
    Case 16
      Call LDA_mem(f4mem(PC + 1))
    Case 32
      Call LDA_imm(f4mem(PC + 1))
    Case 64
      Call LDA_PC
    Case 128
      Call STA_mem(f4mem(PC + 1))
   Case 256
      Call STA_PC
 End Select

End Sub

Public Sub Reset()
'Reset to initial vector, keep memory
  A = 0
  PC = 0
  V = False
  C = 0
End Sub






'Instruction set is here....



Private Sub Add_Amem(operand As Long)
  PC = PC + 2
  If (A + f4mem(operand)) > 65535 Then V = True Else V = False
  A = (A + f4mem(operand)) Mod 65536
  C = C + 4
End Sub

Private Sub Add_Aimm(operand As Long)
  PC = PC + 2
  If (A + operand) > 65535 Then V = True Else V = False
  A = (A + operand) Mod 65536
  C = C + 3
End Sub

Private Sub Add_APC()
  PC = PC + 2
  If (A + PC) > 65535 Then V = True Else V = False
  A = (A + PC) Mod 65536
  C = C + 3
End Sub

Private Sub BVS(operand As Long)
  If V = True Then PC = operand Else PC = PC + 2
  C = C + 3
End Sub

Private Sub LDA_mem(operand As Long)
  PC = PC + 2
  A = f4mem(operand)
  C = C + 3
End Sub

Private Sub LDA_imm(operand As Long)
  PC = PC + 2
  A = operand
  C = C + 3
End Sub

Private Sub LDA_PC()
  PC = PC + 2
  A = PC
  C = C + 3
End Sub

Private Sub STA_mem(operand As Long)
  PC = PC + 2
  f4mem(operand) = A
  C = C + 3
End Sub

Private Sub STA_PC()
  PC = A
  C = C + 3
End Sub
  
'
'
'
'End of instruction set

